/*
 * Copyright (c) 2019, Damir Demirović <damir.demirovic@untz.ba>
 * All rights reserved.
 *
 * This program is free software: you can use, modify and/or
 * redistribute it under the terms of the GNU General Public
 * License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later
 * version. You should have received a copy of this license along
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
@mainpage Mean shift implementation
@author Damir Demirović  damir.demirovic@untz.ba

Mean shift algorithm represent a general non-parametric mode finding procedure.  It's a hill-climbing algorithm on the density defined by a finite mixture or a kernel density estimate. Mean shift can be used as a non-parametric clustering method, for object tracking, image segmentation.
*/

#include <stdio.h>
#include <cstdlib>
#include "ms/ms.h"
#include "io_png/io_png.h"

using namespace std;



/**
 * @file meanshift.cpp
 * @brief Main program for Meanshift segmentation
 *
 * @author Damir Demirović <damir.demirovic@untz.ba>
 */


int main(int argc, char* argv[])
{
    // initial values
    double ms_shift = 5; // Initial value of shift
    int num_iters = 100; // Initial number of iterations for Meanshift

    size_t width, height;
    const int spatial_radius = atoi(argv[2]); // Spatial radius for Meanshift algorithm
    const double color_radius = atof(argv[3]); // Range radius for Meanshift algorithm
    const int minRegion = atoi(argv[4]); // Minimal region for merging
    const string filename_segment = argv[5]; // Filename for segmented image

    // Read image to be segmented
    uchar * image = io_png_read_u8_rgb(argv[1], &width, &height);

    int **ilabels = GenerateLabels(width, height);
    MeanShift(image, ilabels, width, height, spatial_radius, color_radius, minRegion, ms_shift, num_iters);
    //Save segmented image to file
    io_png_write_u8(filename_segment.c_str(), image, width, height, 3);
    delete ilabels;

    free (image);


    return 0;
}
